const gulp = require('gulp'),
    git = require('gulp-git'),
    bump = require('gulp-bump'),
    runSequence = require('run-sequence'),
    log = require('fancy-log'),
    zip = require('gulp-zip'),
    sass = require('gulp-sass'),
    minifyCss = require('gulp-clean-css'),
    rename = require('gulp-rename'),
    through2 = require('through2'),
    autoprefixer = require('gulp-autoprefixer'),
    pxtorem = require('gulp-pxtorem'),
    argv = require('minimist')(process.argv.slice(2));

const browserify = require('browserify'),
    babel = require('babelify'),
    esLint = require('gulp-eslint'),
    uglify = require('gulp-uglify'); // Minifies JS files

const source = require('vinyl-source-stream'),
    buffer = require('vinyl-buffer'),
    sourcemaps = require('gulp-sourcemaps'); // Maps code in a compressed file (E.g. style.css) back to it’s original position in a source file;

// Sass build
sass.compiler = require('node-sass');

const production = Boolean(argv.production);

const jsFiles = [
    { src: "assets/js/admin/admin.js", name: "admin" },
    { src: "assets/js/article/article.js", name: "article" },
    { src: "assets/js/head/head.js", name: "head" },
    { src: "assets/js/body/body.js", name: "body" }
];
const jsDestination = "assets/dist/js";
const sassFiles = "assets/scss/**/*.scss";
const cssFiles = "assets/dist/css";

gulp.task('sass', function () {
  return gulp.src(sassFiles)
    .pipe(sass().on('error', sass.logError))
    .pipe(autoprefixer({
      browsers: ['last 2 versions'],
      cascade: false
    }))
    .pipe(pxtorem())
    .pipe(production ? minifyCss({
        compatibility: 'ie9',
        level: 2,
    }) : through2({objectMode: true}))
    .pipe(production ? rename({
        suffix: '.min',
    }) : through2({objectMode: true}))
    .pipe(gulp.dest(cssFiles));
});

// Watch without rem and minimize etc.
gulp.task('sass-watch', function () {
  return gulp.src(sassFiles)
    .pipe(sass().on('error', sass.logError))
    .pipe(autoprefixer({
      browsers: ['last 2 versions'],
      cascade: false
    }))
    .pipe(production ? minifyCss({
        compatibility: 'ie9',
        level: 2,
    }) : through2({objectMode: true}))
    .pipe(production ? rename({
        suffix: '.min',
    }) : through2({objectMode: true}))
    .pipe(gulp.dest(cssFiles));
});

gulp.task('js', function () {
    jsFiles.forEach(function ({ src, name }) {
        jsTask(src, name);
    });
});

gulp.task('default', function() {
    runSequence('sass');
    runSequence('js');
});

// Watch files
gulp.task('watch', function () {
  gulp.watch(sassFiles, ['sass-watch']);
  jsFiles.forEach(function ({ src, name }) {
    gulp.watch(src, function() {
        jsTask(src, name);
    });
  });
});

function jsTask(src, name) {
    // Lint
    gulp.src(src)
        .pipe(esLint())
        .pipe(esLint.format())
        .pipe(production
            ? esLint.failAfterError()
            : through2({ objectMode: true }));

    // Build
    return browserify({
        entries: src,
        debug: !production,
    })
        .transform(babel.configure({
            presets: ['@babel/preset-env']
        }))
        .bundle()
        .pipe(source(name + (production ? '.min.js' : '.js')))
        .pipe(buffer())
        .pipe(production ? uglify() : through2(
            {objectMode: true}))
        .pipe(!production ? sourcemaps.init({loadMaps: true}) : through2(
            {objectMode: true}))
        .pipe(!production ? sourcemaps.write('.') : through2(
            {objectMode: true}))
        .pipe(gulp.dest(jsDestination));
}


// To get fresh version and avoid cached result
let getPackageJson = () => require('./package.json');

/** ===============
 *  Bump
 *  ===============
 */
gulp.task('bump', function () {
    runSequence('bump:patch-task', 'bumpCommit', 'gitTag');
});
gulp.task('bump:patch', function () {
    runSequence('bump:patch-task', 'bumpCommit', 'gitTag');
});
gulp.task('bump:hotfix', function () {
    runSequence('bump:patch-task', 'bumpCommit', 'gitTag');
});
gulp.task('bump:patch-task', function () {
    return bumpApplication('patch');
});

gulp.task('bump:minor', function () {
    runSequence('bump:minor-task', 'bumpCommit', 'gitTag');
});
gulp.task('bump:minor-task', function () {
    return bumpApplication('minor');
});

gulp.task('bump:major', function () {
    runSequence('bump:major-task', 'bumpCommit', 'gitTag');
});
gulp.task('bump:major-task', function () {
    return bumpApplication('major');
});

function bumpApplication(type) {
    type = type === 'minor' || type === 'major' ? type : 'patch';

    return gulp.src([
        'package.json',
        'package-lock.json',
        'composer.json'
    ])
        .pipe(bump({type: type}).on('error', log))
        .pipe(gulp.dest('./'));
}


gulp.task('bumpCommit', function () {
    return gulp.src([
        'package.json',
        'package-lock.json',
        'composer.json'
    ])
        .pipe(git.add())
        .pipe(git.commit('Release v' + getPackageJson().version))
        .pipe(gulp.dest('./'));
});

gulp.task('gitTag', function () {
    return git.tag(getPackageJson().version, 'Release v' + getPackageJson().version, function (err) {
        if (err) throw err;
    });
});

gulp.task('dist', function () {
    return gulp.src([
        './**/*',
        '!./node_modules',
        '!./node_modules/**',
        '!./vendor',
        '!./vendor/**',
        '!./dist',
        '!./dist/**',
    ])
        .pipe(zip(getPackageJson().vendor + '-' + getPackageJson().name + '-' + getPackageJson().version + '.zip'))
        .pipe(gulp.dest('dist'));
});
